/**	G4blData.java
 *
 *	Downloads the Geant4 data files from CERN. Un-zips and un-tars
 *	each file directly from the web.
 *
 *	Requires the tar classes from ant-1.8.2.jar (org.apache.tools.tar.*).
 *
 *	G4blGui implements a special URL in the Help.html that runs the G4blData
 *	program: file:/G4blData .
 **/

import java.io.*;
import java.net.URL;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import org.apache.tools.tar.*;
import java.util.zip.GZIPInputStream;

public class G4blData extends WindowAdapter implements ActionListener, Runnable {

    static boolean verbose = false;
    static String g4bl_dir = null;
    static String destination = null;
    static JLabel current_stat=null;

    JFrame frame=null;
    JTextField baseURLField=null;
    JTextField destinationField=null;
    String baseURL = "http://geant4.cern.ch/support/source/";
    Vector<String> urls = new Vector<String>();
    Vector<JCheckBox> cbs = new Vector<JCheckBox>();
    Vector<JLabel> stat = new Vector<JLabel>();
    Vector<String> labels = new Vector<String>();
    Vector<String> dirs = new Vector<String>();
    JButton button1=null;
    JButton button2=null;

    public G4blData() {
	// this is for Geant4.9.5
	// For each data set, need an entry in labels, urls, and dirs.
	labels.add("Needed by all physics lists (101MB):");
	urls.add("PhotonEvaporation.2.2.tar.gz");
	dirs.add("PhotonEvaporation2.2");
	labels.add(null);
	urls.add("G4EMLOW.6.23.tar.gz");
	dirs.add("G4EMLOW6.23");
	labels.add("Needed by ABLA lists (0.5MB):");
	urls.add("G4ABLA.3.0.tar.gz");
	dirs.add("G4ABLA3.0");
	labels.add("Needed by specialized hardonic lists (35MB):");
	urls.add("G4PII.1.3.tar.gz");
	dirs.add("G4PII1.3");
	labels.add(null);
	urls.add("G4RadioactiveDecay.3.4.tar.gz");
	dirs.add("RadioactiveDecay3.4");
	labels.add("Needed by optical photons (21MB):");
	urls.add("RealSurface.1.0.tar.gz");
	dirs.add("RealSurface1.0");
	labels.add("Needed by _HP physics lists (1.4GB):");
	urls.add("G4NEUTRONXS.1.1.tar.gz");
	dirs.add("G4NEUTRONXS1.1");
	labels.add(null);
	urls.add("G4NDL.4.0.tar.gz");
	dirs.add("G4NDL4.0");

	// find G4BL_DIR
	g4bl_dir = Util.findG4blDir();
	if(g4bl_dir == null) {
		JOptionPane.showMessageDialog(null, 
				"Cannot determine install directory.\n");
		System.exit(1);
	}

	// find Geant4Data dir (destination)
	destination = Util.findGeant4Data(true);
	if(destination == null) {
		destination = System.getProperty("user.home") + "/Geant4Data";
	}

	// Make sure that baseURL ends with "/"
	if(!baseURL.endsWith("/")) baseURL = baseURL + "/";

	//build up the application window
        frame = new JFrame("G4blData");
	String icon = g4bl_dir + "/share/g4beamline/G4blDataIcon.png";
	if(new File(icon).isFile())
		frame.setIconImage(Toolkit.getDefaultToolkit().getImage(icon));
	frame.addWindowListener(this);

	Box content = Box.createVerticalBox();
	content.add(Box.createVerticalStrut(10));

	Box row = Box.createHorizontalBox();
	JLabel title = new JLabel(" G4blData - Download data files for G4beamline ");
	title.setFont(title.getFont().deriveFont(Font.BOLD,24));
	row.add(title);
	content.add(row);

	content.add(new JLabel(" "));

	row = Box.createHorizontalBox();
	row.add(Box.createHorizontalStrut(10));
	JTextArea instr = new JTextArea(4,50);
	instr.setMargin(new Insets(10,10,10,10));
	instr.append("This program will download data files from CERN and unpack them.\n");
	instr.append("Normally the BaseURL and Destination should be left unchanged,\n");
	instr.append("but change Destination if you already have the data installed.\n");
	instr.append("The physics lists used in your simulations determine which data sets are required.\n");
	instr.append("If you're not sure which you need, and have the space, download them all.");
	instr.setEditable(false);
	row.add(instr);
	row.add(Box.createHorizontalStrut(10));
	content.add(row);

	content.add(new JLabel(" "));

	row = Box.createHorizontalBox();
	row.add(Box.createHorizontalStrut(10));
	row.add(new JLabel("    Base URL:"));
	baseURLField = new JTextField(40);
	baseURLField.setText(baseURL);
	row.add(baseURLField);
	row.add(Box.createHorizontalStrut(10));
	content.add(row);

	row = Box.createHorizontalBox();
	row.add(Box.createHorizontalStrut(10));
	row.add(new JLabel("Destination:"));
	destinationField = new JTextField(40);
	destinationField.setText(destination);
	destinationField.setEditable(false);
	row.add(destinationField);
	JButton but = new JButton("Change");
	but.addActionListener(this);
	row.add(but);
	row.add(Box.createHorizontalStrut(10));
	content.add(row);

	Box col = Box.createVerticalBox();
	for(int i=0; i<urls.size(); ++i) {
		if(labels.get(i) != null) {
			col.add(new JLabel(" "));
			row = Box.createHorizontalBox();
			row.add(new JLabel(labels.get(i)));
			row.add(Box.createHorizontalGlue());
			col.add(row);
		}
		row = Box.createHorizontalBox();
		row.add(Box.createHorizontalStrut(40));
		cbs.add(new JCheckBox(urls.get(i)));
		row.add(cbs.get(i));
		row.add(Box.createHorizontalGlue());
		row.add(Box.createHorizontalStrut(10));
		stat.add(new JLabel(" "));
		row.add(stat.get(i));
		col.add(row);
	}
	row = Box.createHorizontalBox();
	row.add(Box.createHorizontalStrut(30));
	row.add(col);
	row.add(Box.createHorizontalGlue());
	row.add(Box.createHorizontalGlue());
	row.add(Box.createHorizontalGlue());
	row.add(Box.createHorizontalGlue());
	row.add(Box.createHorizontalGlue());
	content.add(row);

	content.add(new JLabel(" "));

	row = Box.createHorizontalBox();
	row.add(Box.createHorizontalGlue());
	button1 = new JButton("Download");
	button1.addActionListener(this);
	row.add(button1);
	button2 = new JButton("Close");
	button2.addActionListener(this);
	row.add(button2);
	row.add(Box.createHorizontalGlue());
	content.add(row);
	content.add(Box.createVerticalStrut(10));

	frame.getContentPane().add(content,BorderLayout.PAGE_START);

        // Display the window.
        frame.pack();
        frame.setVisible(true);

	// save destination and initialize stat[]
	saveDestination();
	scanDestination();

	// update g4setenv.sh, for an old-style data directory
	File dest = new File(destination);
	if(dest.isDirectory()) {
		File g4setenv = new File(destination+"/g4setenv.sh");
		if(!g4setenv.exists())
			Util.writeGeant4DataSetenv(destination);
		JOptionPane.showMessageDialog(frame,"Found Geant4Data in\n    "+
			dest.getAbsolutePath()+"\nYou may not need to download anything.");
	}
    }

    public void saveDestination() {
	// create .data
	try {
	    PrintStream out=new PrintStream(g4bl_dir+"/.data");
	    out.print(destination + "\n");
	    out.close();
	} catch(Exception e) { 
		JOptionPane.showMessageDialog(frame,
		  "INTERNAL ERROR: Cannot write "+g4bl_dir+
		  "/.data\nDestination selection may not be permanent.");
	}
    }

    public void scanDestination() {
	for(int i=0; i<stat.size(); ++i) {
		File f = new File(destination + "/" + dirs.get(i));
		if(f.isDirectory()) {
			cbs.get(i).setSelected(false);
			stat.get(i).setText("Present");
		} else {
			cbs.get(i).setSelected(true);
			stat.get(i).setText("");
		}
	}
    }

    public void windowClosing(WindowEvent event) {
    	System.exit(0);
    }

    // all listener actions come here
    public void actionPerformed(ActionEvent event) {
        String eventName = event.getActionCommand();
	if(eventName == "Download") {
		destination = destinationField.getText();
		baseURL = baseURLField.getText();
		if(!baseURL.endsWith("/")) baseURL = baseURL + "/";
		new Thread(this).start(); // executes run() in a new thread
	} else if(eventName == "Change") { // change destination
		String instr = "The usual places to put Geant4Data are in " +
		    "your home\ndirectory or the install directory:\n        " +
		    System.getProperty("user.home") +
		    "/Geant4Data\n        " +
		    g4bl_dir + "/Geant4Data\nYou also can use some other " +
		    "directory (e.g. because\nthe data are already " +
		    "downloaded there).";
		String s = (String)JOptionPane.showInputDialog(
				frame,
				instr,
				"Change Destination",
				JOptionPane.PLAIN_MESSAGE,
				null,
				null,
				destination);
		if(s != null) {
			destination = s;
			destinationField.setText(destination);
			saveDestination();
			scanDestination();
			System.err.printf("Destination set to '%s'\n",s);
		}
	} else if(eventName == "Cancel") {
		System.exit(0);
	} else if(eventName == "Close") {
		System.exit(0);
	} else {
		System.err.printf("Unknown action '%s'\n",eventName);
	}
    }

    public void run() {
	button2.setText("Cancel");
	downloadFiles();
	button2.setText("Close");
    }

    public void downloadFiles() {
	System.out.println("Writing data to directory '"+destination+"'");
	File dir = new File(destination);
	if(!dir.isDirectory()) {
		if(!dir.mkdir()) {
			System.err.printf("Cannot create directory '%s'\n",
				destination);
			JOptionPane.showMessageDialog(frame,
			  "Cannot create directory '"+destination+"'.");
			return;
		}
	}
    	for(int i=0; i<urls.size(); ++i) {
		if(cbs.get(i).isSelected()) {
			String url = baseURL + urls.get(i);
			System.out.println("downloading "+url);
			current_stat = stat.get(i);
			current_stat.setText("0");
			try { untarURL(url); } catch(Exception e) { }
			cbs.get(i).setSelected(false);
		}
	}

	// create g4setenv.sh
	Util.writeGeant4DataSetenv(destination);
    }

    public static void untarFile(String path) throws Exception {
	File tarFile = new File(path);
	// 1) check for valid file
	if (!tarFile.isFile()) {
		throw new Exception("Cannot open file '"+path+"'.");
	}

	untarInputStream(new FileInputStream(tarFile),path.endsWith("z"));
    }

    public static void untarURL(String url) throws Exception {
    	URL tarURL = new URL(url);
	InputStream stream=null;
	try {
		stream = tarURL.openStream();
	} catch(Exception e) {
		throw new Exception("Cannot open URL '"+url+"'.");
	}
	try {
		untarInputStream(stream,url.endsWith("z"));
	} catch(Exception e) {
		if(stream != null) stream.close();
		throw e;
	}
    }

    // Files and directories from the tar stream are put into the destination directory.
    public static void untarInputStream(InputStream stream, boolean zip) throws Exception {
        TarInputStream inputStream=null;
        OutputStream outputStream=null;
	long totalBytes = 0;
	long printBytes = 0;
	int nErrors=0;
	try {
	    if(zip)
	    	inputStream = new TarInputStream(new GZIPInputStream(stream));
	    else
	    	inputStream = new TarInputStream(stream);
	    // For each entry in the tar, extract and save the entry to the file system
	    TarEntry entry;
	    while (null != (entry = inputStream.getNextEntry())) {
		// for each entry to be extracted
		int bytesRead;
		// if the entry is a directory, create the directory
		if (entry.isDirectory()) {
			File fileOrDir = new File(destination+"/"+entry.getName());
			fileOrDir.mkdirs();
			if(!fileOrDir.isDirectory()) {
				++nErrors;
				System.err.printf("ERROR: cannot create directory '%s'\n",
					destination+"/"+entry.getName());
			}
			if(verbose) System.out.println("Created dir " + entry.getName());
			continue;
		}
		// write to file
		byte[] buf = new byte[4096];
		outputStream = new FileOutputStream(destination+"/"+entry.getName());
		if(outputStream == null) {
			++nErrors;
			System.err.printf("ERROR: cannot create file '%s'\n",
				destination+"/"+entry.getName());
		} else try {
			while ((bytesRead = inputStream.read(buf, 0, buf.length)) > -1) {
				outputStream.write(buf, 0, bytesRead);
				totalBytes += bytesRead;
				if(totalBytes > printBytes+10000) {
					printBytes = totalBytes;
					current_stat.setText(""+totalBytes);
				}
			}
			if(verbose) System.out.println("Extracted " + entry.getName());
		} catch(Exception e) {
			++nErrors;
			System.err.printf("ERROR: while writing '%s'\n",
				destination+"/"+entry.getName());
		} finally {
			outputStream.close();
		}
	    }// while
	} catch (Exception e) {
		e.printStackTrace();
		throw new Exception("An error occurred while extracting file. Error " + e.getMessage());
	} finally { // close I/O streams
		if (inputStream != null)
			try { inputStream.close(); } catch (IOException e) { }
		if (outputStream != null)
			try { outputStream.close(); } catch (IOException e) { }
	}
	if(nErrors == 0)
	    	current_stat.setText("Complete.");
	else
		current_stat.setText("Errors: "+nErrors);
    }

    public static void main(String[] args) throws Exception {
	//untarURL(args[0]);
	new G4blData();
    }
};
