/**	Util.java
 *
 *	Utility routines related to directories for G4blGui and G4blData.
 **/

import java.io.*;
import java.net.URL;
import java.util.*;
import javax.swing.*;

public class Util {

static String osName = null;
static String g4bl_dir = null;
static String geant4_data = null;

/// findG4blDir() returns the absolute path of the G4beamline install directory.
/// Returns null if it cannot be determined.
public static String findG4blDir() {
	if(g4bl_dir != null) return g4bl_dir;

	osName = System.getProperty("os.name");
	String filesep = System.getProperty("file.separator");
	String pathsep = System.getProperty("path.separator");
	g4bl_dir = System.getenv("G4BL_DIR");
	if(g4bl_dir == null || g4bl_dir == "") {
		g4bl_dir = System.getProperty("java.class.path");
		int i = g4bl_dir.indexOf(pathsep);
		if(i >= 0) g4bl_dir = g4bl_dir.substring(0,i);
		File tmp = new File(g4bl_dir);
		g4bl_dir = tmp.getParentFile().getParentFile().getAbsolutePath();
	}
	if(g4bl_dir.endsWith("/") || g4bl_dir.endsWith("\\"))
		g4bl_dir = g4bl_dir.substring(0,g4bl_dir.length()-1);

	// verify g4bl_dir
	File dir = new File(g4bl_dir);
	String ext = "";
	if(osName.startsWith("Windows")) ext = ".exe";
	if(!new File(g4bl_dir+filesep+"bin"+filesep+"g4beamline"+ext).exists())
		g4bl_dir = null;
	System.out.printf("G4BL_DIR=%s\n",g4bl_dir);
	return g4bl_dir;
}

/// findGeant4Data() will find the directory and return its absolute path.
/// Returns null if not found.
public static String findGeant4Data(boolean justName) {
	if(geant4_data != null) return geant4_data;

	if(findG4blDir() == null) return null;

	// read $G4BL_DIR/.data, if it exists
	File d = new File(g4bl_dir);
	if(!d.isDirectory()) return null;
	d = new File(d,".data");
	if(d.exists()) {
		try {
			FileReader fr = new FileReader(d);
			BufferedReader br = new BufferedReader(fr);
			geant4_data = br.readLine();
			br.close();
if(geant4_data != null) System.out.printf(".data contains '%s'\n",geant4_data);
			if(justName && geant4_data != null) return geant4_data;
			d = new File(geant4_data);
			d = new File(d,"g4setenv.sh");
			if(!d.exists())
				geant4_data = null;
		} catch(Exception e) { }
	}
if(geant4_data != null) System.out.printf("Geant4Data found in '%s'\n",geant4_data);

	return geant4_data;
}

/// Writes g4setenv.sh into destination, listing just the datasets that are present
// NOTE: fails silently.
public static void writeGeant4DataSetenv(String destination) {
	try {
	    File dir = new File(destination);
	    if(!dir.isDirectory()) return;
	    PrintStream out = new PrintStream(destination+"/g4setenv.sh");
	    out.print("# g4setenv.sh - must be sourced by a bash-compatible shell.\n");
	    out.print("DIR=`dirname \"${BASH_SOURCE[0]}\"`\n");
	    out.print("DIR=`cd \"$DIR\" >&- 2>&- && pwd`\n");
	    out.print("CYGPATH=echo\n");
	    out.print("if type cygpath >&- 2>&-; then CYGPATH=\"cygpath -m\"; fi\n");
	    out.print("function setdata { export $1=\"$($CYGPATH \"$DIR/$2\")\"; }\n");

	    String[] list = dir.list();
	    for(int j=0; j<list.length; ++j) {
		if(list[j].startsWith("G4ABLA"))
	    	    out.print("setdata G4ABLADATA "+list[j]+"\n");
		if(list[j].startsWith("G4EMLOW"))
	    	    out.print("setdata G4LEDATA "+list[j]+"\n");
		if(list[j].startsWith("G4NDL"))
	    	    out.print("setdata G4NEUTRONHPDATA "+list[j]+"\n");
		if(list[j].startsWith("G4NEUTRONX"))
	    	    out.print("setdata G4NEUTRONXSDATA "+list[j]+"\n");
		if(list[j].startsWith("G4PII"))
	    	    out.print("setdata G4PIIDATA "+list[j]+"\n");
		if(list[j].startsWith("PhotonEvaporation"))
	    	    out.print("setdata G4LEVELGAMMADATA "+list[j]+"\n");
		if(list[j].startsWith("RadioactiveDecay"))
	    	    out.print("setdata G4RADIOACTIVEDATA "+list[j]+"\n");
		if(list[j].startsWith("RealSurface"))
	    	    out.print("setdata G4REALSURFACEDATA "+list[j]+"\n");
	    }
		out.close();
	    } catch(Exception e) { }
}

/// reads dir/g4setenv.sh, putting entries into the env Map.
/// displays environment variables to stdout.
/// Note: we don't have to worry about cygpath here, because Cygwin is
/// never involved (only Java and G4beamline ever see this environment).
public static void readGeant4DataSetenv(String dir, Map<String,String> env) {
	try {
		FileReader fr = new FileReader(dir+"/g4setenv.sh");
		BufferedReader br = new BufferedReader(fr);
		String l;
		while((l=br.readLine()) != null) {
			String[] w = l.split(" ");
			if(!w[0].equals("setdata")) continue;
			env.put(w[1],dir+"/"+w[2]);
			System.out.print(w[1]+"="+dir+"/"+w[2]+"\n");
		}
		br.close();
	} catch(Exception e) { 
		JOptionPane.showConfirmDialog(null,"Cannot read " + dir +
			"/g4setenv.sh");
	}
}

};


